
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.ArrayList;

/**
 * this class is about to get a search query from the user and sends it to the server.
 * after sending the query to the server this method waits for query results.
 * for each search query one instant of this is made.
 *
 * @author Naeem Esfahani,Kamyar Rafati
 * @version 1, 0
 * @see Client
 */
public class Search extends Thread {

    /**
     * This is the array which contains the results of the query which comes from the server.
     */
    private ArrayList resultlist;

    /**
     * this array contains the address of each result which is used for downloading.
     */
    private ArrayList resultaddresslist;

    /**
     * this array contains the port address of each result which is used for downloading.
     */
    private ArrayList resultportlist;

    /**
     * the searching query.
     */
    private String query;


    /**
     * the address of the server for making connections.
     */
    private String serveraddress;

    /**
     * the socket which is used to connect to server.
     */
    private Socket server = null;
    private ObjectInputStream input;
    private ObjectOutputStream output;

    /**
     * the port of the server for connecting to that.
     */
    private int port;

    /**
     * Construct a new Search Object.
     *
     * @param query         the searching query.
     * @param serveraddress the socket which is used to connect to server.
     * @param port          the socket which is used to connect to server.
     * @param list          This is the array which contains the results of the query which comes from the server.
     * @param addresses     this array contains the address of each result which is used for downloading.
     * @param ports         this array contains the port address of each result which is used for downloading.
     */
    public Search(String query, String serveraddress, int port, ArrayList list, ArrayList addresses, ArrayList ports) {
        // Initialing fiels.
        this.query = query;
        this.serveraddress = serveraddress;
        this.port = port;
        resultlist = list;
        resultaddresslist = addresses;
        resultportlist = ports;
    }

    /**
     * this method connects to the server and send the search query and recieve the results.
     */
    public void run() {
        // Make Socket Connection to server.
        try {
            server = new Socket(serveraddress, port);
        } catch (IOException e) {
            System.out.println("Error : Cannot Connect to Server !!");
            return;
        }
        try {
            output = new ObjectOutputStream(server.getOutputStream());
            input = new ObjectInputStream(server.getInputStream());
        } catch (IOException e) {
            end();
            return;
        }

        try {
            // hand shaking with server.
            output.writeObject("Hello New Search");
            output.reset();

            // sendinq search query.
            output.writeObject(query);
            output.reset();

            // recieving search results from server.
            try {
                ResponsePacket result = (ResponsePacket) input.readObject();

                while (result != null) {
                    System.out.println("Client : Wait to Read Results");
                    System.out.println("New Result Set From Address: " + result.getAddress());
                    System.out.println(result.getArray().toString());
                    while (!result.getArray().isEmpty()) {
                        String temp = (String) result.getArray().remove(0);

                        synchronized (resultlist) {
                            resultlist.add(temp);
                        }
                        synchronized (resultaddresslist) {
                            resultaddresslist.add(result.getAddress());
                        }
                        synchronized (resultportlist) {
                            resultportlist.add(new Integer(result.getPort()));
                        }
                    }
                    result = (ResponsePacket) input.readObject();
                }
            } catch (IOException e) {
                System.out.println("Error : Cannot Read from Socket in Search !!");
                end();
                return;
            } catch (ClassNotFoundException e) {
                System.out.println("Error : Cannot Read from Socket in Search !!");
                end();
                return;
            }


        } catch (IOException e) {
            System.out.println("Error : Cannot Write in Socket in Search !!");
            end();
            return;
        }

    }

    /**
     * this method is responsible for finalizing this class.
     */
    private void end() {
        //closing connections
        try {
            input.close();
            output.close();
            server.close();
            System.out.println("Client : Closing Socket in Search");

        } catch (IOException e) {
            System.out.println("Error : Closing Search Connection in Client !!");
        }
    }

}
