
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * this class waits for connections for download requests and then make Upload object.
 * this class make the ability of parallel downloading for different client.
 *
 * @author Naeem Esfahani,Kamyar Rafati
 * @version 1, 0
 * @see Client
 * @see Search
 * @see Upload
 */
public class WaitForUploadConnection extends Thread {

    /**
     * this is the port which this thread listen to.
     */
    private int port;

    /**
     * the server socket which listen.
     */
    private ServerSocket connection;

    /**
     * the falg that shows the end of the operation.
     */
    private boolean ended;

    /**
     * the path of the sharing directory.
     */
    private String path;

    /**
     * make a new instant of this class
     *
     * @param port this is the port which this thread listen to.
     * @param path the path of the sharing directory.
     */
    public WaitForUploadConnection(int port, String path) {
        // initialing fields.
        this.path = path;
        this.port = port;
        ended = false;
    }

    /**
     * this method waits for incoming connections and make Upload objects.
     */
    public void run() {
		try {
			connection = new ServerSocket(port);
		} catch (IOException e) {
			System.out.println("Client : Cannot Open Socket For Waiting for Upload Request");
			return;
		}

        while (!ended) {
            Socket client = null;
            // waiting for incoming connections for upload.
            try {
                client = connection.accept();
            } catch (IOException e) {
                System.out.println("Client : Cannot Open Socket For Waiting for Upload Request");
                return;
            }

            // making new Object for uploading.
            Upload newClient = new Upload(client, path);

            newClient.start();
        }


    }

    /**
     * this method is used to finalize object:
     * closing connection and etc.
     */
    void end() {
        ended = true;

        //closing connections.
        try {
            connection.close();
        } catch (IOException e) {
            System.out.println("Client : Cannot close download socket in waitforuploadconnection");
        }
    }
}
