
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.ArrayList;

/**
 * This some part of server that for each new connected client we have
 * a queue and a responsible thread to send the requests that queued into
 * that queue to related client.
 *
 * @author Naeem & Kamyar
 */
public class SendQuery extends Thread {
    /**
     * The other end of this socket is client and receives or queries
     */
    private Socket client;

    /**
     * Queue of the requestes to be sent to the client
     */
    private Queue queue;

    /**
     * List of the server side Queues that a request goes to all of them
     */
    ArrayList queuesList;

    /**
     * An outout stream over client
     */
    ObjectOutputStream output;

    /**
     * An input stream over client
     */
    ObjectInputStream input;

    /**
     * For ease of implementation
     */
    RequestPacket requestpacket;

    /**
     * Shows if run must be ended
     */
    boolean ended;

    public SendQuery(Socket client, ArrayList queuesList, ObjectInputStream input, ObjectOutputStream output) {
        this.client = client;
        this.queuesList = queuesList;
        this.input = input;
        this.output = output;
        ended = false;
        queue = new Queue();

        synchronized (queuesList) {
            queuesList.add(queue);
        }
    }

    /**
     * @see Thread#run()
     */
    public void run() {

        while (!ended) {
            //get something from queue
            if (queue.isEmpty()) {
                try {
                    synchronized (queue) {
                        queue.wait();
                    }
                } catch (InterruptedException e) {
                    end();
                    return;
                }
            }

            //This request must be sent
            requestpacket = (RequestPacket) queue.get();

            try {
                output.writeObject(requestpacket.getRequest());
                output.reset();

                System.out.println("Server : Sending new Query to Clients!!");

                //Gets this from client
                ArrayList result = (ArrayList) input.readObject();
                //Get the port from client
                Integer port = (Integer) input.readObject();
                requestpacket.getWaitQueue().put(new ResponsePacket(result, client.getInetAddress(), port.intValue()));

                synchronized (requestpacket.getWaitQueue()) {
                    requestpacket.getWaitQueue().notifyAll();
                }

            } catch (IOException e) {
                System.out.println("Error : Reading from Client Socket !!");
                end();
                return;
            } catch (ClassNotFoundException e) {
                System.out.println("Error : No Class Definition Found !!");
                end();
                return;
            }
        }
    }

    /**
     * Finish it up that the thread stops
     */
    public void end() {
        ended = true;
        //Wait queue understand that must be ended
        requestpacket.getWaitQueue().put(null);

        synchronized (requestpacket.getWaitQueue()) {
            requestpacket.getWaitQueue().notifyAll();
        }

        synchronized (queuesList) {
            System.out.println("Server : Removing from Queue");
            queuesList.remove(queue);
        }

        try {
            client.close();
            System.out.println("Server : Closing Socket in End-SendQuery");
        } catch (IOException e) {
            System.out.println("Error : Closing Client Socket !!");
        }
    }
}
