
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.ArrayList;

/**
 * This thread is created every time a search request from a client comes.
 * the answer of that request is queued into its waiting queue.
 *
 * @author Naeem & Kamyar
 */
public class WaitForQuery extends Thread {
    /**
     * The other end of this socket is client
     */
    private Socket client;

    /**
     * List of server send query queues
     */
    private ArrayList queuesList;

    /**
     * Requested pattern
     */
    private String request;

    /**
     * The input stream on client
     */
    private ObjectInputStream input;

    /**
     * The output stream over client
     */
    private ObjectOutputStream output;

    /**
     * The queue that results come into it
     */
    private Queue waitQueue;

    public WaitForQuery(Socket client, ArrayList queuesList, ObjectInputStream input, ObjectOutputStream output) {
        this.client = client;
        this.queuesList = queuesList;
        request = null;
        waitQueue = new Queue();
        this.input = input;
        this.output = output;
    }

    /**
     * @see Thread#run()
     */
    public void run() {

        if (!init())
            return;

        //Number of interactive clients
        int count = 0;
        RequestPacket requestpacket = new RequestPacket(request, waitQueue);

        synchronized (queuesList) {
            for (int i = 0; i < queuesList.size(); i++) {
                synchronized (queuesList.get(i)) {
                    //Filling receive query queues
                    ((Queue) queuesList.get(i)).put(requestpacket);
                }

                //I most wait for one more response
                count++;

                synchronized (queuesList.get(i)) {
                    queuesList.get(i).notifyAll();
                }
            }
        }

        //Wait for all the results
        while (count > 0) {
            if (waitQueue.isEmpty()) {
                try {
                    synchronized (waitQueue) {
                        waitQueue.wait();
                    }
                } catch (InterruptedException e) {
                    continue;
                }
            }

            //One of the results has arrived
            count--;

            try {
                synchronized (waitQueue) {
                    System.out.println("Server : Waiting for catching results " + count);
                    ResponsePacket j = (ResponsePacket) waitQueue.get();

                    //If that thread has not been ended
                    if (j != null) {
                        output.writeObject(j);
                        output.reset();
                    }
                }
            } catch (IOException e) {
                System.out.println("Error : Writing In Socket !!");
            }
        }
        try {
            System.out.println("Server : Results Finished ");
            //It has finnished
            output.writeObject(null);
            output.reset();
        } catch (IOException e) {
            System.out.println("Error : Writing In Socket !!");
        }

        try {
            input.close();
            output.close();
            client.close();
        } catch (IOException e) {
            System.out.println("Error : In Closing Socket & Streams !!");
        }
    }

    /**
     * Gets request from client
     *
     * @return if the request arrival has problem
     */
    private boolean init() {
        try {
            request = (String) input.readObject();
            System.out.println("Server : Receiving new Query !!");
        } catch (IOException e) {
            System.out.println("Error : Socket Problem While Waiting For Query !!");
            return false;
        } catch (ClassNotFoundException e) {
            System.out.println("Error : Invalid Object In Query !!");
            return false;
        }

        return true;
    }
}
